--- v1/fl_font_xft.cxx	2007-10-12 16:15:01.000000000 +0100
+++ v2/fl_font_xft.cxx	2007-10-12 16:35:44.000000000 +0100
@@ -125,19 +125,44 @@
 }
 
 static XftFont* fontopen(const char* name, bool core) {
-  fl_open_display();
-  int slant = XFT_SLANT_ROMAN;
-  int weight = XFT_WEIGHT_MEDIUM;
-  // may be efficient, but this is non-obvious
-  switch (*name++) {
-  case 'I': slant = XFT_SLANT_ITALIC; break;
-  case 'P': slant = XFT_SLANT_ITALIC;
-  case 'B': weight = XFT_WEIGHT_BOLD; break;
-  case ' ': break;
-  default: name--;
+  // Check: does it look like we have been passed an old-school XLFD fontname?
+  bool is_xlfd = false;
+  int hyphen_count = 0;
+  int comma_count = 0;
+  unsigned len = strlen(name);
+  if (len > 512) len = 512; // ensure we are not passed an unbounded font name
+  for(unsigned idx = 0; idx < len; idx++) {
+    if(name[idx] == '-') hyphen_count++; // check for XLFD hyphens
+    if(name[idx] == ',') comma_count++;  // are there multiple names?
   }
-  // this call is extremely slow...
-  return XftFontOpen(fl_display, fl_screen,
+  if(hyphen_count >= 14) is_xlfd = true; // Not a robust check, but good enough?
+
+  fl_open_display();
+
+  if(!is_xlfd) { // Not an XLFD - open as a XFT style name
+    int slant = XFT_SLANT_ROMAN;
+    int weight = XFT_WEIGHT_MEDIUM;
+
+    /* This "converts" FLTK-style font names back into "regular" names, extracting
+     * the BOLD and ITALIC codes as it does so - all FLTK font names are prefixed
+     * by 'I' (italic) 'B' (bold) 'P' (bold italic) or ' ' (regular) modifiers.
+     * This gives a fairly limited font selection ability, but is retained for
+     * compatability reasons. If you really need a more complex choice, you are best
+     * calling Fl::set_fonts(*) then selecting the font by font-index rather than by
+     * name anyway. Probably.
+     * If you want to load a font who's name does actually begin with I, B or P, you
+     * MUST use a leading space OR simply use lowercase for the name...
+     */
+    /* This may be efficient, but it is non-obvious. */
+    switch (*name++) {
+    case 'I': slant = XFT_SLANT_ITALIC; break; // italic
+    case 'P': slant = XFT_SLANT_ITALIC;        // bold-italic (falls-through)
+    case 'B': weight = XFT_WEIGHT_BOLD; break; // bold
+    case ' ': break;                           // regular
+    default: name--;                           // no prefix, restore name
+    }
+    // this call is extremely slow...
+    return XftFontOpen(fl_display, fl_screen,
 		     XFT_FAMILY, XftTypeString, name,
 		     XFT_WEIGHT, XftTypeInteger, weight,
 		     XFT_SLANT, XftTypeInteger, slant,
@@ -146,7 +171,18 @@
 		     core ? XFT_CORE : 0, XftTypeBool, true,
 		     XFT_RENDER, XftTypeBool, false,
 		     (void *)0);
-}
+  }
+  else { // We were passed a font name in XLFD format
+    char *local_name = strdup(name);
+    if(comma_count) { // This means we were passed multiple XLFD's
+      char *pc = strchr(local_name, ',');
+      *pc = 0; // terminate the XLFD at the first comma
+    }
+    XftFont *the_font = XftFontOpenXlfd(fl_display, fl_screen, local_name);
+    free(local_name);
+   return the_font;
+  }
+} // end of fontopen
 
 Fl_FontSize::Fl_FontSize(const char* name) {
   encoding = fl_encoding_;
